# AUTOMATICALLY GENERATED by https://github.com/tqdm/shtab



complete -c streamlink -e
complete -c streamlink -f

complete -c streamlink -n "__fish_use_subcommand" -s h -l help -d '
            Show this help message and exit.
        '
complete -c streamlink -n "__fish_use_subcommand" -s V -l version -d '
            Show version string and exit.
        '
complete -c streamlink -n "__fish_use_subcommand" -l version-check -d '
            Run a version check and exit.
        '
complete -c streamlink -n "__fish_use_subcommand" -l auto-version-check -d '
            Enable or disable the automatic check for a new version of Streamlink.

            Default is "no".
        '
complete -c streamlink -n "__fish_use_subcommand" -l config -d '
            Load options from this config file.

            Can be repeated to load multiple files, in which case the options are
            merged on top of each other where the last config has highest priority.
        '
complete -c streamlink -n "__fish_use_subcommand" -l no-config -d '
            Disable loading any default or custom config files.
        '
complete -c streamlink -n "__fish_use_subcommand" -l locale -d '
            Override the system'"'"'s locale setting, for selecting the preferred subtitle and audio language.

            The locale is formatted as `[language_code]_[country_code]`, e.g. `en_US` or `es_ES`.

            Default is system locale.
        '
complete -c streamlink -n "__fish_use_subcommand" -s l -l loglevel -rka "none critical error warning info debug trace all" -d '
            Set the log message threshold.

            Valid levels are, in order of increasing verbosity:

            `none`, `critical`, `error`, `warning`, `info`, `debug`, `trace`, `all`

            Default is "info".
        '
complete -c streamlink -n "__fish_use_subcommand" -l logformat -d '
            Set a custom logging format.

            See the Python standard library'"'"'s `logging.Formatter` docs for more information about the logging format
            and the available `LogRecord` attributes. Streamlink'"'"'s formatter uses the curly brace style.

            The default format depends on the chosen log level.

            For verbose levels (`trace` and `all`):

            Default is "[{asctime}][{threadName}][{name}][{levelname}] {message}".

            Otherwise:

            Default is "[{name}][{levelname}] {message}".
        '
complete -c streamlink -n "__fish_use_subcommand" -l logdateformat -d '
            Set a custom logging date format.

            This formats the `LogRecord`'"'"'s `asctime` attribute via `strftime()`.

            The default date format depends on the chosen log level.

            For verbose levels (`trace` and `all`):

            Default is "%%H:%%M:%%S.%%f".

            Otherwise:

            Default is "%%H:%%M:%%S".
        '
complete -c streamlink -n "__fish_use_subcommand" -l logfile -d '
            Append log output to `FILE` instead of writing to stdout/stderr.

            User prompts and download progress won'"'"'t be written to `FILE`.

            A value of `-` (dash) will set the file name to an ISO8601-like string
            and will choose the following default log directories.

            Windows:

              %%TEMP%%\streamlink\logs

            macOS:

              ${HOME}/Library/Logs/streamlink

            Linux/BSD:

              ${XDG_STATE_HOME:-${HOME}/.local/state}/streamlink/logs
        '
complete -c streamlink -n "__fish_use_subcommand" -s Q -l quiet -d '
            Suppress all console and log output, and also disable user prompts.
        '
complete -c streamlink -n "__fish_use_subcommand" -s j -l json -d '
            Output JSON representations instead of the normal text output.

            Useful for external scripting.
        '
complete -c streamlink -n "__fish_use_subcommand" -l plugins -d '
            Print a list of all currently installed plugins.
        '
complete -c streamlink -n "__fish_use_subcommand" -l show-matchers -d '
            Show the list of matchers of a specific plugin (URL regex pattern with opt. priority and opt. name).

            The output is a human-readable pseudo YAML format. Please use --json when reading matcher data programmatically.
        '
complete -c streamlink -n "__fish_use_subcommand" -l can-handle-url -d '
            Check if Streamlink has a plugin that can handle the specified URL.

            Status code is `0` on success, `1` on failure.

            Useful for external scripting.
        '
complete -c streamlink -n "__fish_use_subcommand" -l can-handle-url-no-redirect -d '
            Same as --can-handle-url, but without following redirects when looking up the URL.
        '
complete -c streamlink -n "__fish_use_subcommand" -l no-plugin-cache -d '
            Disable I/O of the plugin key-value store.

            If disabled, plugins won'"'"'t be able to load or store data like cookies, authentication data, etc.
            The data which is loaded or stored depends on each plugin implementation.
        '
complete -c streamlink -n "__fish_use_subcommand" -l no-plugin-sideloading -d '
            Disable automatic sideloading of third-party plugins from the default location.

            See the plugin-sideloading documentation for where third-party plugins are loaded from.
        '
complete -c streamlink -n "__fish_use_subcommand" -l plugin-dir -d '
            Load additional plugins from this directory.

            Can be set multiple times to load plugins from multiple directories.
        '
complete -c streamlink -n "__fish_use_subcommand" -l plugin-dirs -d '
            Load additional plugins from a list of comma-separated directories. (deprecated)
        '
complete -c streamlink -n "__fish_use_subcommand" -l interface -d '
            Select the network interface, either by IP address or by interface name.

            The following optional prefixes are supported:

            - `if!<name>` interface name
            - `host!<name>` IP address or hostname
            - `ifhost!<interface>!<host>` interface name and IP address or hostname

            Note: On Windows, only IP addresses are supported and prefixes are ignored.
        '
complete -c streamlink -n "__fish_use_subcommand" -s 4 -l ipv4 -d '
            Resolve address names to IPv4 only. This option overrides --ipv6.
        '
complete -c streamlink -n "__fish_use_subcommand" -s 6 -l ipv6 -d '
            Resolve address names to IPv6 only. This option overrides --ipv4.
        '
complete -c streamlink -n "__fish_use_subcommand" -s p -l player -d '
            Set the player executable that will be launched (unless a different output method was chosen).

            Either set an absolute or relative path to the player executable, or just set the executable'"'"'s name
            if it can be resolved from the paths of the system'"'"'s `PATH` environment variable.

            In addition to setting the player executable path, custom player arguments can be set via --player-args.

            Note: In the past, --player allowed defining additional player arguments, which as a consequence required wrapping
            player paths that contained spaces in quotation marks. This is unsupported since release `6.0.0`.

            Default is VLC player, if available.
        '
complete -c streamlink -n "__fish_use_subcommand" -s a -l player-args -d '
            Set a string of custom --player launch arguments that will be parsed and tokenized.

            The value can contain formatting variables surrounded by curly braces, `{` and `}`.
            Curly brace characters can be escaped by doubling, e.g. `{{` and `}}`.

            Available formatting variables:

            `{playerinput}`
                This is the input argument that the --player will receive. For standard input (stdin),
                it is `-` (dash), but it can also be a file path or URL, depending on the options used.
                If unset, then the player input argument will be appended to the parsed player arguments list.

            `{playertitleargs}`
                The automatically generated player title arguments, if a supported --player was found. See --title for more.
                If unset, automatically generated player title arguments will be prepended to the parsed player arguments list.

            Example:

              %(prog)s -p vlc -a "--play-and-exit --no-one-instance" <url> [stream]

            Default is "".
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-env -d '
            Add an additional environment variable to the spawned --player process, in addition to the ones inherited from
            the Streamlink/Python parent process. This allows setting player environment variables in config files.

            Can be repeated to add multiple environment variables.
        '
complete -c streamlink -n "__fish_use_subcommand" -s v -l player-verbose -d '
            Write the --player'"'"'s stdout/stderr output to Streamlink'"'"'s stdout/stderr output.
        '
complete -c streamlink -n "__fish_use_subcommand" -l verbose-player -d '
            Deprecated in favor of --player-verbose.
        '
complete -c streamlink -n "__fish_use_subcommand" -s n -l player-fifo -d '
            Make the --player read the stream through a named pipe instead of the stdin pipe.
        '
complete -c streamlink -n "__fish_use_subcommand" -l fifo -d '
            Deprecated in favor of --player-fifo.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-http -d '
            Make the --player read the stream through HTTP instead of the stdin pipe.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-continuous-http -d '
            Make the --player read the stream through HTTP, but unlike --player-http,
            it will continuously try to open the stream if the player requests it.

            This enables the handling of stream disconnects if the player is
            capable of reconnecting to a HTTP stream. This is usually done by
            setting the player to a "repeat mode".
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-external-http -d '
            Serve stream data through HTTP without opening the --player. This is
            useful to allow external devices like smartphones or streaming boxes to
            watch streams they wouldn'"'"'t be able to otherwise.

            The default behavior is similar to the --player-continuous-http option,
            but no player program will be started, and the server will listen on all available
            connections instead of just in the local (loopback) interface.

            See --player-external-http-interface for choosing a specific network interface, and
            see --player-external-http-port for choosing a non-randomized port.

            Optionally, the --player-external-http-continuous option allows for disabling
            the continuous run-mode, so that Streamlink will stop when the stream ends.

            The URLs that can be used to access the stream will be printed to the
            console, and the server can be interrupted using CTRL-C.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-external-http-continuous -d '
            Set the run-mode of --player-external-http to continuous or non-continuous.

            In the continuous run-mode, Streamlink will keep running after the stream has ended
            and will wait for the next HTTP request being made unless it gets shut down via CTRL-C.

            If set to non-continuous, Streamlink will stop once the stream has ended.

            Default is true.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-external-http-interface -d '
            Set the network interface on which the HTTP server will be listening on.
            If unset or set to `0.0.0.0`, all available interfaces will be bound.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-external-http-port -d '
            Set the port of the external HTTP server if that mode is enabled.
            Omit or set to `0` to use a random high ( >1024) port.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-passthrough -d '
            A comma-delimited list of stream types to pass to the --player as a URL to
            let it handle the transport of the stream instead of Streamlink.

            Stream types that can be converted into a playable URL are:

            hls, http

            Make sure the player can handle the stream type when using this.
        '
complete -c streamlink -n "__fish_use_subcommand" -l player-no-close -d '
            By default, Streamlink will close the --player when the stream ends.
            This is to avoid "dead" GUI players lingering after Streamlink has exited.

            It does however have the side-effect of sometimes closing a
            player before it has played back all of its cached data.

            This option will instead let the player decide when to exit.
        '
complete -c streamlink -n "__fish_use_subcommand" -s t -l title -d '
            Change the title of the --player'"'"'s window.

            Please see the "Metadata variables" section of Streamlink'"'"'s CLI documentation for all available metadata variables,
            as well as the "Plugins" section for the list of metadata variables defined in each plugin.

            Only the following players are supported:
            
            - mpv
            - PotPlayer
            - VLC

            Example:

                %(prog)s -p mpv --title "{author} - {category} - {title}" <URL> [STREAM]
        '
complete -c streamlink -n "__fish_use_subcommand" -s O -l stdout -d '
            Write stream data to `stdout` instead of playing it in the --player.
        '
complete -c streamlink -n "__fish_use_subcommand" -s o -l output -d '
            Write stream data to `FILENAME` instead of playing it in the --player.
            If `FILENAME` is set to `-` (dash), then the stream data will be written to `stdout`,
            similar to the --stdout argument.

            Directories and subdirectories will be created if they do not exist, if filesystem permissions allow.

            Unless --force is set, Streamlink will ask for confirmation before writing if `FILENAME` already exists.

            Please see the "Metadata variables" section of Streamlink'"'"'s CLI documentation for all available metadata variables,
            as well as the "Plugins" section for the list of metadata variables defined in each plugin.

            Unsupported characters in substituted variables will be replaced with an underscore.

            Example:

                %(prog)s --output "~/recordings/{author}/{category}/{id}-{time:%%Y%%m%%d%%H%%M%%S}.ts" <URL> [STREAM]
        '
complete -c streamlink -n "__fish_use_subcommand" -s r -l record -d '
            Write stream data to `FILENAME` while at the same time allowing playback in the --player or writing it to --stdout.
            If `FILENAME` is set to `-` (dash), then the stream data will be written to `stdout`,
            similar to the --stdout argument, while still opening the player.

            Directories and subdirectories will be created if they do not exist, if filesystem permissions allow.

            Unless --force is set, Streamlink will ask for confirmation before writing if `FILENAME` already exists.

            Please see the "Metadata variables" section of Streamlink'"'"'s CLI documentation for all available metadata variables,
            as well as the "Plugins" section for the list of metadata variables defined in each plugin.

            Unsupported characters in substituted variables will be replaced with an underscore.

            Example:

                %(prog)s --record "~/recordings/{author}/{category}/{id}-{time:%%Y%%m%%d%%H%%M%%S}.ts" <URL> [STREAM]
        '
complete -c streamlink -n "__fish_use_subcommand" -s R -l record-and-pipe -d '
            Deprecated in favor of --stdout --record=FILENAME.
        '
complete -c streamlink -n "__fish_use_subcommand" -l fs-safe-rules -rka "POSIX Windows" -d '
            The rules used to make formatting variables filesystem-safe are chosen
            automatically according to the type of system in use. This overrides
            the automatic detection.

            Intended for use when Streamlink is running on a UNIX-like OS but writing
            to Windows filesystems such as NTFS; USB devices using VFAT or exFAT; CIFS
            shares that are enforcing Windows filename limitations, etc.

            These characters are replaced with an underscore for the rules in use:

            - POSIX: `\x00-\x1F /`
            - Windows: `\x00-\x1F \x7F " * / : < > ? \ |`
        '
complete -c streamlink -n "__fish_use_subcommand" -s f -l force -d '
            When using --output or --record, always write to file even if it already exists (overwrite).
        '
complete -c streamlink -n "__fish_use_subcommand" -l skip -d '
            When using --output or --record, never write to file if it already exists (don'"'"'t prompt).

            Takes precedence over --force.
        '
complete -c streamlink -n "__fish_use_subcommand" -l progress -rka "yes force no" -d '
            When using --output or --record, show or hide the download progress bar, or force it if there'"'"'s no terminal.

            Default is yes.
        '
complete -c streamlink -n "__fish_use_subcommand" -l url -d '
            A URL to attempt to extract streams from.

            Usually, the protocol of http(s) URLs can be omitted (`https://`),
            depending on the implementation of the plugin being used.

            This is an alternative to setting the URL using a positional argument and can be useful if set in a config file.
        '
complete -c streamlink -n "__fish_use_subcommand" -l default-stream -d '
            Stream to play.

            Use `best` or `worst` for selecting the highest or lowest available quality.

            Fallback streams can be specified by using a comma-separated list:

              "720p,480p,best"

            This is an alternative to setting the stream using a positional argument and can be useful if set in a config file.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-url -d '
            If possible, translate the resolved stream to a URL and print it.
        '
complete -c streamlink -n "__fish_use_subcommand" -l retry-streams -d '
            Retry fetching the list of available streams until streams are found
            while waiting `DELAY` second(s) between each attempt.

            The number of retry attempts can be capped with --retry-max.
            A default value of ``1`` is implied for non-zero values of --retry-max.

            If both --retry-streams and --retry-max are set to `0`, then only one attempt will be made
            to fetch the list of available streams. This is the default behavior.
        '
complete -c streamlink -n "__fish_use_subcommand" -l retry-max -d '
            Stop fetching the list of available streams after `COUNT` retry attempt(s).

            A value of `0` makes Streamlink fetch streams indefinitely if --retry-streams is set to a non-zero value.
            If --retry-streams is unset, then the default delay between fetching available streams is 1 second.
        '
complete -c streamlink -n "__fish_use_subcommand" -l retry-open -d '
            After a successful fetch, try `ATTEMPTS` time(s) to open the stream until giving up.

            Default is 1.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-types -l stream-priority -d '
            A comma-delimited list of stream types to allow.

            The order will be used to separate streams when there are multiple
            streams with the same name but different stream types. Any stream type
            not listed will be omitted from the available streams list.  An `*` (asterisk) can
            be used as a wildcard to match any other type of stream, e.g. dash.

            Default is "hls,http,*".
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-sorting-excludes -d '
            Fine-tune the `best` and `worst` stream name synonyms by excluding unwanted streams.

            If all of the available streams get excluded, `best` and `worst` will become
            inaccessible and new special stream synonyms `best-unfiltered` and `worst-unfiltered`
            can be used as a fallback selection method.

            The filter-expression'"'"'s format is:

              [operator]<value>

            Valid operators are `>`, `>=`, `<` and `<=`. If no operator is specified then
            equality is tested.

            For example this will exclude streams ranked higher than "480p":

              --stream-sorting-excludes ">480p"

            Multiple filters can be used by separating each expression with a comma.

            For example this will exclude streams from two quality types:

              --stream-sorting-excludes ">480p,>medium"

        '
complete -c streamlink -n "__fish_use_subcommand" -l ringbuffer-size -d '
            The maximum size of the ringbuffer.

            Mebibytes or kibibytes (base 2) can be specified via the M or K suffix respectively.

            The ringbuffer is used as a temporary storage between the stream and the player.
            This allows Streamlink to download the stream faster than the player which reads the data from the ringbuffer.

            The smaller the size of the ringbuffer, the higher the chance of the player buffering if the download speed
            decreases, and the higher the size, the more data can be use as a storage to recover from volatile download speeds.

            Most players have their own additional cache and will read the ringbuffer'"'"'s content as soon as data is available.
            If the player stops reading data while playback is paused, Streamlink will continue to download the stream in the
            background as long as the ringbuffer doesn'"'"'t get full.

            Default is "16M".
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-segment-attempts -d '
            The number of download attempts of each stream segment before giving up.

            This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.

            Default is 3.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-segment-threads -d '
            The size of the thread pool used to download segments. Minimum value is `1` and maximum is `10`.

            This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.

            Default is 1.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-segment-timeout -d '
            The maximum time to wait for each segment to start downloading.

            This applies to all different kinds of segmented stream types, such as DASH, HLS, etc.

            Default is 10.0.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-segmented-duration -d '
            Limit the output duration of segmented streams, like HLS and DASH.
            The actual duration may be slightly longer, as it is rounded to the nearest segment.

            Set to 0 to disable.

            Default is 0.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-segmented-queue-deadline -d '
            A multiplication factor of the time frame in which new segments must be queued in order to prevent playback issues
            due to lack of video/audio data. If this segment-queue-deadline has not been met, the stream will be stopped early.

            The intention of this segment-queue-deadline is to be able to stop early when the end of a stream is not announced
            by the server, so Streamlink doesn'"'"'t have to wait until a buffer read-timeout occurs. See --stream-timeout.

            The base time this multiplication factor is applied to depends on the specific
            stream implementation and the respective values returned by the streaming server.
            This deadline check is done after trying to fetch new data.

            Set to ``0`` to disable.

            Default is 3.0.

            By default, wait three times as long for new segments to be made available than the server'"'"'s advertised time frame.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-timeout -d '
            The maximum time to wait for an unfiltered stream to continue outputting data.

            This applies to all different kinds of stream types, such as DASH, HLS, HTTP, etc.

            Default is 60.0.
        '
complete -c streamlink -n "__fish_use_subcommand" -l stream-passthrough-encrypted -d '
            Pass through data from encrypted streams without decryption or encryption checks.

            This applies to DASH and HLS streams, and will likely result in garbage output.
        '
complete -c streamlink -n "__fish_use_subcommand" -l mux-subtitles -d '
            Automatically mux available subtitles into the output stream.

            Needs to be supported by the used plugin.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-live-edge -d '
            Number of segments from the live stream'"'"'s current live position to begin streaming.
            The size or length of each segment is determined by the streaming provider.

            Lower values will decrease the latency, but will also increase the chance of buffering, as there is less time for
            Streamlink to download segments and write their data to the output buffer. The number of parallel segment downloads
            can be set with --stream-segment-threads and the HLS playlist reload time to fetch and queue new segments can be
            overridden with --hls-playlist-reload-time.

            Default is 3.

            Note: During live playback, the caching/buffering settings of the used player will add additional latency.
            To adjust this, please refer to the player'"'"'s own documentation for the required configuration.
            Player parameters can be set via --player-args.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-segment-stream-data -d '
            Immediately write segment data into output buffer while downloading.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-playlist-reload-attempts -d '
            The maximum number of attempts when reloading the HLS playlist before giving up.

            Default is 3.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-playlist-reload-time -d '
            Set a custom HLS playlist reload time value, either in seconds or by using one of the following keywords:

            - segment: The duration of the last segment in the current playlist
            - live-edge: The sum of segment durations of the live edge value minus one
            - default: The playlist'"'"'s target duration metadata

            Default is default.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-segment-queue-threshold -d '
            Deprecated in favor of --stream-segmented-queue-deadline.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-segment-ignore-names -d '
            A comma-delimited list of segment names that will get filtered out.

            Example: --hls-segment-ignore-names 000,001,002

            This will ignore every segment that ends with 000.ts, 001.ts and 002.ts

            Default is None.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-segment-key-uri -d '
            Override the segment encryption key URIs for encrypted streams.

            The value can be templated using the following variables, which will be
            replaced with their respective part from the source segment URI:

              {url} {scheme} {netloc} {path} {query}

            Examples:

              --hls-segment-key-uri "https://example.com/hls/encryption_key"
              --hls-segment-key-uri "{scheme}://1.2.3.4{path}{query}"
              --hls-segment-key-uri "{scheme}://{netloc}/custom/path/to/key"

            Default is None.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-audio-select -d '
            Select one or more specific audio sources by language code or name.
            Can be set to `*` (asterisk) to include all audio sources.

            Examples:

              --hls-audio-select "English,German"
              --hls-audio-select "en,de"
              --hls-audio-select "*"

            Note: This is only useful in special circumstances where the regular
            locale option fails, such as when multiple sources of the same language exist.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-start-offset -d '
            The amount of time to skip from the beginning of the stream.
            For live streams, this is a negative offset from the end of the stream (rewind).

            Default is 0.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-duration -d '
            Deprecated in favor of --stream-segmented-duration.
        '
complete -c streamlink -n "__fish_use_subcommand" -l hls-live-restart -d '
            Skip to the beginning of a live stream, or as far back as possible.
        '
complete -c streamlink -n "__fish_use_subcommand" -l dash-manifest-reload-attempts -d '
            The maximum number of attempts when reloading the DASH manifest before giving up.

            Default is 3.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-ffmpeg -d '
            Set the location of the FFmpeg executable if it can'"'"'t be resolved
            from the paths of the system'"'"'s `PATH` environment variable.

            FFmpeg is required to access or mux separate video and audio streams,
            e.g. in DASH streams or HLS streams with multiple sources.

            Example: --ffmpeg-ffmpeg "/usr/local/bin/ffmpeg"
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-no-validation -d '
            Disable FFmpeg validation and version logging.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-verbose -d '
            Write FFmpeg'"'"'s stderr output to Streamlink'"'"'s stderr output.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-verbose-path -d '
            Write FFmpeg'"'"'s stderr output to PATH.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-loglevel -d '
            Change FFmpeg'"'"'s `-loglevel` value to `LOGLEVEL`.

            Unless --ffmpeg-verbose or --ffmpeg-verbose-path is set, changing the log level won'"'"'t have any effect.

            Default is "info".
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-fout -d '
            Set the output format to `OUTFORMAT`. This only applies to streams which require muxing.

            Default is "matroska".

            Example: --ffmpeg-fout "mpegts"
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-video-transcode -d '
            Transcode the video to `CODEC`. This only applies to streams which require muxing.

            Default is "copy".

            Example: --ffmpeg-video-transcode "h264"
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-audio-transcode -d '
            Transcode the audio to `CODEC`. This only applies to streams which require muxing.

            Default is "copy".

            Example: --ffmpeg-audio-transcode "aac"
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-copyts -d '
            Set the `-copyts` FFmpeg option, so input timestamps won'"'"'t be processed
            and the initial start time offset value be kept.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-start-at-zero -d '
            Enable the `-start_at_zero` FFmpeg option when using --ffmpeg-copyts.
        '
complete -c streamlink -n "__fish_use_subcommand" -l ffmpeg-validation-timeout -d '
            Timeout in seconds for FFmpeg version validation.

            Default is 4.0.

            Increase this on low-power systems if FFmpeg startup is slow.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-proxy -d '
            An HTTP proxy to use for all HTTP and HTTPS requests, including WebSocket connections.

            Example: --http-proxy "http://hostname:port/"
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-cookie -d '
            A cookie to add to each HTTP request.

            Can be repeated to add multiple cookies.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-cookies-file -d '
            A path to a cookies file whose cookie data will be added to HTTP requests.

            Can be repeated to add multiple cookie files.

            The file format must adhere to the Netscape HTTP Cookie File format (MozillaCookieJar, curl, etc.),
            also known as cookies.txt. Be aware that this format loses information about RFC 2965 cookies,
            and also about newer or non-standard cookie-attributes such as port.

            Unlike --http-cookie, which sends data with every request, this option enables granular control
            by respecting domain, path, and other cookie attributes.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-header -d '
            A header to add to each HTTP request.

            Can be repeated to add multiple headers.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-query-param -d '
            A query parameter to add to each HTTP request.

            Can be repeated to add multiple query parameters.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-ignore-env -d '
            Ignore HTTP settings set in the environment, such as environment variables (`HTTP_PROXY`, etc)
            or `~/.netrc` authentication.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-no-ssl-verify -d '
            Don'"'"'t attempt to verify TLS/SSL certificates.

            Use with caution, as it has TLS/SSL security implications.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-disable-dh -d '
            Disable Diffie Hellman key exchange.

            Use with caution, as it has TLS/SSL security implications.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-ssl-cert -d '
            SSL certificate to use: a .pem file.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-ssl-cert-crt-key -d '
            SSL certificate to use: a .crt and a .key file.
        '
complete -c streamlink -n "__fish_use_subcommand" -l http-timeout -d '
            Set the general timeout value used by all HTTP requests except the ones covered by other options.

            Default is 20.0.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser -d '
            Enable or disable support for Streamlink'"'"'s webbrowser API.

            Streamlink'"'"'s webbrowser API allows plugins which implement it to launch a web browser and extract data from websites
            which they otherwise couldn'"'"'t do via the regular HTTP session in Python due to specific JavaScript restrictions.

            The web browser is run isolated and in a clean environment without access to regular user data.

            Streamlink currently only supports Chromium-based web browsers using the Chrome Devtools Protocol (CDP).
            This includes Chromium itself, Google Chrome, Microsoft Edge, Brave, Vivaldi, and others, but full support for
            third party Chromium forks is not guaranteed. Please try Chromium or Google Chrome when encountering any issues.

            Default is true.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-executable -d '
            Path to the web browser'"'"'s executable.

            By default, it is looked up automatically according to the rules of the used webbrowser API implementation.
            This usually involves a list of known executable names and fallback paths on all supported operating systems.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-timeout -d '
            The maximum amount of time which the web browser can take to launch and execute.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-cdp-host -d '
            Host for the web browser'"'"'s inter-process communication interface (CDP specific).

            Default is 127.0.0.1.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-cdp-port -d '
            Port for the web browser'"'"'s inter-process communication interface (CDP specific).

            Tries to find a free port by default.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-cdp-timeout -d '
            The maximum amount of time for waiting on a single CDP command response.
        '
complete -c streamlink -n "__fish_use_subcommand" -l webbrowser-headless -d '
            Whether to launch the web browser in headless mode or not.
            When enabled, it stays completely hidden and doesn'"'"'t require a desktop environment to run.

            Please be aware that headless mode might be blocked by websites which implement bot detections.

            Default is false.
        '
complete -c streamlink -n "__fish_use_subcommand" -l bbciplayer-username -d 'The username used to register with bbc.co.uk.'
complete -c streamlink -n "__fish_use_subcommand" -l bbciplayer-password -d 'A bbc.co.uk account password to use with --bbciplayer-username.'
complete -c streamlink -n "__fish_use_subcommand" -l bbciplayer-hd -d 'Prefer HD streams over local SD streams, some live programmes may not be broadcast in HD.'
complete -c streamlink -n "__fish_use_subcommand" -l clubbingtv-username -d 'The username used to register with Clubbing TV.'
complete -c streamlink -n "__fish_use_subcommand" -l clubbingtv-password -d 'A Clubbing TV account password to use with --clubbingtv-username.'
complete -c streamlink -n "__fish_use_subcommand" -l kick-low-latency -d '
        Enables low latency streaming by prefetching HLS segments.
        Sets --hls-segment-stream-data to true and --hls-live-edge to 2, if it is higher.
        Reducing --hls-live-edge to `1` will result in the lowest latency possible, but will most likely cause buffering.

        In order to achieve true low latency streaming during playback, the player'"'"'s caching/buffering settings will
        need to be adjusted and reduced to a value as low as possible, but still high enough to not cause any buffering.
        This depends on the stream'"'"'s bitrate and the quality of the connection to Kick'"'"'s servers. Please refer to the
        player'"'"'s own documentation for the required configuration. Player parameters can be set via --player-args.

        Note: Low latency streams have to be enabled by the broadcasters on Kick themselves.
        Regular streams can cause buffering issues with this option enabled due to the reduced --hls-live-edge value.
    '
complete -c streamlink -n "__fish_use_subcommand" -l niconico-email -d 'The email or phone number associated with your Niconico account'
complete -c streamlink -n "__fish_use_subcommand" -l niconico-password -d 'The password of your Niconico account'
complete -c streamlink -n "__fish_use_subcommand" -l niconico-user-session -d '
        Value of the user-session token.

        Can be used as an alternative to providing a password.
    '
complete -c streamlink -n "__fish_use_subcommand" -l niconico-purge-credentials -d 'Purge cached Niconico credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l niconico-timeshift-offset -d '
        Amount of time to skip from the beginning of a stream.

        Default is 0.
    '
complete -c streamlink -n "__fish_use_subcommand" -l openrectv-email -d 'The email associated with your openrectv account, required to access any openrectv stream.'
complete -c streamlink -n "__fish_use_subcommand" -l openrectv-password -d 'An openrectv account password to use with --openrectv-email.'
complete -c streamlink -n "__fish_use_subcommand" -l pixiv-sessionid -d 'The pixiv.net sessionid that'"'"'s used in pixiv'"'"'s PHPSESSID cookie.'
complete -c streamlink -n "__fish_use_subcommand" -l pixiv-devicetoken -d 'The pixiv.net device token that'"'"'s used in pixiv'"'"'s device_token cookie.'
complete -c streamlink -n "__fish_use_subcommand" -l pixiv-purge-credentials -d 'Purge cached Pixiv credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l pixiv-performer -d 'Select a co-host stream instead of the owner stream.'
complete -c streamlink -n "__fish_use_subcommand" -l raiplay-email -d 'The email used to register with raiplay.it.'
complete -c streamlink -n "__fish_use_subcommand" -l raiplay-password -d 'A raiplay.it account password to use with --raiplay-email.'
complete -c streamlink -n "__fish_use_subcommand" -l raiplay-purge-credentials -d 'Purge cached RaiPlay credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l soop-username -d 'The username used to register with sooplive.com.'
complete -c streamlink -n "__fish_use_subcommand" -l soop-password -d 'A sooplive.com account password to use with --soop-username.'
complete -c streamlink -n "__fish_use_subcommand" -l soop-purge-credentials -d 'Purge cached Soop credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l soop-stream-password -d 'The password for the stream.'
complete -c streamlink -n "__fish_use_subcommand" -l steam-email -d 'A Steam account email address to access friends/private streams'
complete -c streamlink -n "__fish_use_subcommand" -l steam-password -d 'A Steam account password to use with --steam-email.'
complete -c streamlink -n "__fish_use_subcommand" -l streann-url -d 'Source URL where the iframe is located, only required for direct URLs of ott.streann.com'
complete -c streamlink -n "__fish_use_subcommand" -l tf1-email -d 'The email address used to register with tf1.fr.'
complete -c streamlink -n "__fish_use_subcommand" -l tf1-password -d 'A tf1.fr account password to use with --tf1-email.'
complete -c streamlink -n "__fish_use_subcommand" -l tf1-purge-credentials -d 'Purge cached tf1.fr credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l twitcasting-password -d 'Password for private Twitcasting streams.'
complete -c streamlink -n "__fish_use_subcommand" -l twitch-low-latency -d '
        Enables low latency streaming by prefetching HLS segments.
        Sets --hls-segment-stream-data to true and --hls-live-edge to 2, if it is higher.
        Reducing --hls-live-edge to `1` will result in the lowest latency possible, but will most likely cause buffering.

        In order to achieve true low latency streaming during playback, the player'"'"'s caching/buffering settings will
        need to be adjusted and reduced to a value as low as possible, but still high enough to not cause any buffering.
        This depends on the stream'"'"'s bitrate and the quality of the connection to Twitch'"'"'s servers. Please refer to the
        player'"'"'s own documentation for the required configuration. Player parameters can be set via --player-args.

        Note: Low latency streams have to be enabled by the broadcasters on Twitch themselves.
        Regular streams can cause buffering issues with this option enabled due to the reduced --hls-live-edge value.
    '
complete -c streamlink -n "__fish_use_subcommand" -l twitch-supported-codecs -d '
        A comma-separated list of codec names which signals Twitch the client'"'"'s stream codec preference.
        Which streams and which codecs are available depends on the specific channel and broadcast.

        Default is "h264".

        Supported codecs are h264, h265 and av1. Set to "h264,h265,av1" to enable all codecs.

        Higher quality streams may only be available by enabling h265 or av1.
        Lower quality streams which are re-encoded on Twitch'"'"'s end may still be h264, even if not requested.
    '
complete -c streamlink -n "__fish_use_subcommand" -l twitch-api-header -d '
        A header to add to each Twitch API HTTP request.

        Can be repeated to add multiple headers.

        Useful for adding authentication data that can prevent ads. See the plugin-specific documentation for more information.
    '
complete -c streamlink -n "__fish_use_subcommand" -l twitch-access-token-param -d '
        A parameter to add to the API request for acquiring the streaming access token.

        Can be repeated to add multiple parameters.
    '
complete -c streamlink -n "__fish_use_subcommand" -l twitch-force-client-integrity -d 'Don'"'"'t attempt requesting the streaming access token without a client-integrity token.'
complete -c streamlink -n "__fish_use_subcommand" -l twitch-purge-client-integrity -d 'Purge cached Twitch client-integrity token and acquire a new one.'
complete -c streamlink -n "__fish_use_subcommand" -l ustream-password -d 'A password to access password protected UStream.tv channels.'
complete -c streamlink -n "__fish_use_subcommand" -l ustvnow-username -d 'Your USTV Now account username'
complete -c streamlink -n "__fish_use_subcommand" -l ustvnow-password -d 'Your USTV Now account password'
complete -c streamlink -n "__fish_use_subcommand" -l wwenetwork-email -d 'The email associated with your WWE Network account, required to access any WWE Network stream.'
complete -c streamlink -n "__fish_use_subcommand" -l wwenetwork-password -d 'A WWE Network account password to use with --wwenetwork-email.'
complete -c streamlink -n "__fish_use_subcommand" -l yupptv-boxid -d 'The yupptv.com boxid that'"'"'s used in the BoxId cookie.'
complete -c streamlink -n "__fish_use_subcommand" -l yupptv-yuppflixtoken -d 'The yupptv.com yuppflixtoken that'"'"'s used in the YuppflixToken cookie.'
complete -c streamlink -n "__fish_use_subcommand" -l yupptv-purge-credentials -d 'Purge cached YuppTV credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l zattoo-email -d 'The email associated with your zattoo account, required to access any zattoo stream.'
complete -c streamlink -n "__fish_use_subcommand" -l zattoo-password -d 'A zattoo account password to use with --zattoo-email.'
complete -c streamlink -n "__fish_use_subcommand" -l zattoo-purge-credentials -d 'Purge cached zattoo credentials to initiate a new session and reauthenticate.'
complete -c streamlink -n "__fish_use_subcommand" -l zattoo-stream-types -d '
        A comma-delimited list of stream types which should be used.

        The following types are allowed: dash, hls7

        Default is "dash".
    '
complete -c streamlink -n "__fish_use_subcommand" -d 'A URL to attempt to extract streams from.'
complete -c streamlink -n "__fish_use_subcommand" -d 'Stream to play.'
